package com.sample;

import org.kie.api.event.process.ProcessCompletedEvent;
import org.kie.api.event.process.ProcessEventListener;
import org.kie.api.event.process.ProcessNodeLeftEvent;
import org.kie.api.event.process.ProcessNodeTriggeredEvent;
import org.kie.api.event.process.ProcessStartedEvent;
import org.kie.api.event.process.ProcessVariableChangedEvent;
import org.kie.api.io.ResourceType;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.manager.RuntimeEnvironment;
import org.kie.api.runtime.manager.RuntimeEnvironmentBuilder;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.internal.io.ResourceFactory;
import org.kie.internal.runtime.manager.context.EmptyContext;
import org.kie.internal.runtime.manager.context.ProcessInstanceIdContext;

/**
 * This is a sample file to launch a process.
 */
public class ProcessTest {

	public static final void main(String[] args) {
		
	
		try {

			// first configure environment that will be used by RuntimeManager

			RuntimeEnvironment environment = RuntimeEnvironmentBuilder.Factory
					.get()
					.newEmptyBuilder()
					.addAsset(ResourceFactory.newClassPathResource("process/Exception.bpmn2"),
							ResourceType.BPMN2)
					.addAsset(ResourceFactory.newClassPathResource("process/Exceptionsubprocess.bpmn2"),
							ResourceType.BPMN2).get();

			// next create RuntimeManager - in this case singleton strategy is chosen
			RuntimeManager manager = RuntimeManagerFactory.Factory.get().newSingletonRuntimeManager(environment);
			// then get RuntimeEngine out of manager - using empty context as singleton does not keep track
			// of runtime engine as there is only one
			RuntimeEngine runtime = manager.getRuntimeEngine(ProcessInstanceIdContext.get());
			// get KieSession from runtime runtimeEngine - already initialized with all handlers, listeners, etc that
			// were configured
			// on the environment

			KieSession ksession = runtime.getKieSession();
			ksession.addEventListener(new ProcessEventListener() {
				
				@Override
				public void beforeVariableChanged(ProcessVariableChangedEvent arg0) {
					// TODO Auto-generated method stub
					
					
				}
				
				@Override
				public void beforeProcessStarted(ProcessStartedEvent arg0) {
					// TODO Auto-generated method stub
					
					
					
				}
				
				@Override
				public void beforeProcessCompleted(ProcessCompletedEvent arg0) {
					// TODO Auto-generated method stub
					
					
				}
				
				@Override
				public void beforeNodeTriggered(ProcessNodeTriggeredEvent arg0) {
					// TODO Auto-generated method stub
					System.out.println("====before=====");
					System.out.println("We are entering node: "+arg0.getNodeInstance().getNodeName());
					System.out.println("This node is located in process : "+arg0.getProcessInstance().getProcessName());
					System.out.println("The process state is : "+arg0.getProcessInstance().getState());
					
					
					
				}
				
				@Override
				public void beforeNodeLeft(ProcessNodeLeftEvent arg0) {
					// TODO Auto-generated method stub
					
				}
				
				@Override
				public void afterVariableChanged(ProcessVariableChangedEvent arg0) {
					// TODO Auto-generated method stub
					
				}
				
				@Override
				public void afterProcessStarted(ProcessStartedEvent arg0) {
					// TODO Auto-generated method stub
					
				}
				
				@Override
				public void afterProcessCompleted(ProcessCompletedEvent arg0) {
					// TODO Auto-generated method stub
					
				}
				
				@Override
				public void afterNodeTriggered(ProcessNodeTriggeredEvent arg0) {
					// TODO Auto-generated method stub
					
					System.out.println("====after=====");
					System.out.println("We have left: "+arg0.getNodeInstance().getNodeName());
					System.out.println("This node is located in process : "+arg0.getProcessInstance().getProcessName());
					System.out.println("The process state is : "+arg0.getProcessInstance().getState());
					
				}
				
				@Override
				public void afterNodeLeft(ProcessNodeLeftEvent arg0) {
					// TODO Auto-generated method stub
					
				}
			});
			
			System.out.println("1 = ACTIVE, 2 = COMPLETED\n=========================");
		   ksession.startProcess("Evaluation.Exception");
//			Long pid = processInstance.getId();
			runtime = manager.getRuntimeEngine(EmptyContext.get());
			ksession = runtime.getKieSession();
			
			
//			for (ProcessInstance p : ksession.getProcessInstances()) {
//				System.out.println("process name: "+p.getProcessName());
//				System.out.println("process state: "+p.getState());
//				
//			}
//			System.out.println("state:" + processInstance.getState());
//			System.out.println("We are going to signal process");
			ksession.signalEvent("SubProcess_Exception", null);
			
//			for (ProcessInstance p : ksession.getProcessInstances()) {
//				System.out.println("process name: "+p.getProcessName());
//				System.out.println("process state: "+p.getState());
//				
//			}
//			System.out.println("name: " + processInstance.getProcessName());
//			System.out.println("state: " + processInstance.getState());
			
			System.out.println("===after sending a signal the main process and sub process should both "
					+ "be ended===");
			for (ProcessInstance p: ksession.getProcessInstances()) {
				
				System.out.println("after signal process name:"+p.getProcessName());
				System.out.println("after signal process state:"+p.getState());
			}
			ksession.dispose();

		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

}
